import base64
import json
import pathlib
from abc import ABC, abstractmethod

import codefast as cf

from vps.meta import (GOST_CONFIG_BASE64, SHADOWSOCKS_CONFIG_BASE64,
                      SUPVISOR_CONF_TEMPLATE, VPS_STATUS_DEMO)
from vps.osi import BashRunner, Cmd
from vps.bash_scripts import UTILS, WARP, PROMPT, VIM_CONFIG


class SupervisorConfig(ABC):
    def run(self):
        try:
            cf.info('update {} supervisor config file'.format(self.app_name))
            cf.io.write(self.conf, self.conf_path)
            cf.info('restart supervisor')
            BashRunner.call(f'supervisorctl update')
        except Exception as e:
            cf.error(e)


class GostConfig(SupervisorConfig):
    def __init__(self):
        self.app_name = 'gost'
        _bash = BashRunner.call('which bash')
        self.command = f'{_bash} /app/gost'
        self.conf = SUPVISOR_CONF_TEMPLATE.replace(
            'PLACEHOLDER', self.app_name)
        self.conf = self.conf.replace('COMMAND', self.command)
        self.conf_path = '/etc/supervisor/conf.d/{}.conf'.format(self.app_name)

        self.init_config_file()

    def init_config_file(self):
        cf.info('init gost config file')
        content = base64.b64decode(GOST_CONFIG_BASE64).decode('utf-8')
        cf.io.write(content, '/app/gost')


class EndlessConfig(SupervisorConfig):
    def __init__(self):
        self.app_name = 'endlessh'
        self.command = BashRunner.get_app_path('endlessh') + ' -p 22'
        self.conf = SUPVISOR_CONF_TEMPLATE.replace(
            'PLACEHOLDER', self.app_name)
        self.conf = self.conf.replace('COMMAND', self.command)
        self.conf_path = '/etc/supervisor/conf.d/{}.conf'.format(self.app_name)


class SystemConfig:
    def __init__(self) -> None:
        self.cmds = ['timedatectl set-timezone Asia/Shanghai', 'mkdir /app']

    def config_vim(self):
        _path = str(pathlib.Path('~/.vimrc').expanduser().resolve())
        cf.io.write(VIM_CONFIG, _path)

    def add_bash_library(self):
        '''Copy some bash scripts to local'''
        cf.io.write(UTILS, '/app/utils.sh')
        cf.io.write(WARP, '/app/warp')

    def config_PS1(self):
        '''Add color to PS1 and `ls` to be colorized.'''
        _path = str(pathlib.Path('~/.bashrc').expanduser().resolve())
        with open(_path, 'a') as f:
            f.write(PROMPT)

    def run(self):
        for cmd in self.cmds:
            str_cmd = Cmd(f'Running {cmd}', cmd, f'{cmd} finished')
            BashRunner.call_with_msg(str_cmd)

        self.add_bash_library()
        self.config_vim()
        self.config_PS1()


class MonitorConfig(SupervisorConfig):
    '''VPS monitor configuration'''

    def __init__(self):
        self.app_name = 'vpsmonitor'
        self.command = '/usr/local/bin/vps_monitor -vps_name {} -alert_time 08-40 -task_types vnstat -interface eth0'.format(
            BashRunner.get_hostname())
        self.conf = SUPVISOR_CONF_TEMPLATE.replace(
            'PLACEHOLDER', self.app_name)
        self.conf = self.conf.replace('COMMAND', self.command)
        self.conf_path = '/etc/supervisor/conf.d/{}.conf'.format(self.app_name)


class StatusMonitorConfig(SupervisorConfig):
    def __init__(self) -> None:
        self.app_name = 'vpsstatus'
        self.command = BashRunner.get_app_path('vpsstatus')
        self.conf = SUPVISOR_CONF_TEMPLATE.replace(
            'PLACEHOLDER', self.app_name)
        self.conf = self.conf.replace('COMMAND', self.command)
        self.conf_path = '/etc/supervisor/conf.d/{}.conf'.format(self.app_name)

        self.init_demo_config()

    def init_demo_config(self):
        """Add a demo config file for vpsstatus"""
        cf.info('init vpsstatus config file')
        _path = str(pathlib.Path(
            '~/.config/vpsstatus.json').expanduser().resolve())
        cf.js.write(VPS_STATUS_DEMO, _path)


class QbittorrentConfig(SupervisorConfig):
    '''Qbittorrent configuration'''

    def __init__(self):
        self.app_name = 'qbittorrent'
        self.command = BashRunner.get_app_path('qbittorrent-nox')
        self.conf = SUPVISOR_CONF_TEMPLATE.replace(
            'PLACEHOLDER', self.app_name)
        self.conf = self.conf.replace('COMMAND', self.command)
        self.conf_path = '/etc/supervisor/conf.d/qbittorrent.conf'


class CalculatePrime(SupervisorConfig):
    '''NSQ consumer configuration'''

    def __init__(self):
        self.app_name = 'primeconsumer'
        self.command = BashRunner.get_app_path('calculate_prime')
        self.conf = SUPVISOR_CONF_TEMPLATE.replace(
            'PLACEHOLDER', self.app_name)
        self.conf = self.conf.replace('COMMAND', self.command)
        self.conf_path = '/etc/supervisor/conf.d/primeconsumer.conf'


class AbstractSystemctl(ABC):
    def __init__(self, name):
        self.name = name

    @abstractmethod
    def run(self):
        pass


class Shadowsocks(AbstractSystemctl):
    def __init__(self):
        super().__init__('shadowsocks')
        _cont = base64.b64decode(SHADOWSOCKS_CONFIG_BASE64).decode('utf-8')
        self._config = json.loads(_cont)
        self._config_path = '/etc/shadowsocks-libev/config.json'

    def run(self):
        cf.info('overwrite config file')
        cf.js.write(self._config, self._config_path)
        cf.info('restart shadowsocks')
        BashRunner.call('systemctl restart shadowsocks-libev')
        cf.warning('please check if the port is openned')
